@@ -7,7 +7,9 @@ describe('36 Lemonldap::NG Manager', function() {
   describe('Diff interface', function() {
 
     it('should find key changed', function() {
-      browser.get('/diff.html#6/7');
+      browser.get('/diff.html#!/5/6');
+            browser.sleep(5000);
+
       element(by.id('t-generalParameters')).click();
       element(by.id('t-advancedParams')).click();
       element(by.id('t-security')).click();
