@@ -1,7 +1,7 @@
 # Main running methods file
 package Lemonldap::NG::Handler::Main::Run;
 
-our $VERSION = '2.0.12';
+our $VERSION = '2.22.0';
 
 package Lemonldap::NG::Handler::Main;
 
@@ -9,8 +9,11 @@ use strict;
 
 #use AutoLoader 'AUTOLOAD';
 use MIME::Base64;
+use URI;
 use URI::Escape;
 use Lemonldap::NG::Common::Session;
+use Lemonldap::NG::Handler::Main::MsgActions;
+use Time::HiRes 'usleep';
 
 # Methods that must be overloaded
 
@@ -26,68 +29,21 @@ sub logout {
     return $class->unlog(@_);
 }
 
-sub status {
-    my $class;
-    $class = $#_ ? shift : __PACKAGE__;
-    return $class->getStatus(@_);
-}
-
 # Public methods
 
-# Return Handler::Lib::Status output
-sub getStatus {
-    my ( $class, $req ) = @_;
-    $class->logger->debug("Request for status");
-    my $statusPipe = $class->tsv->{statusPipe};
-    my $statusOut  = $class->tsv->{statusOut};
-    my $args       = '';
-    if ( $ENV{LLNGSTATUSHOST} ) {
-        require IO::Socket::INET;
-        foreach ( 64322 .. 64331 ) {
-            if ( $statusOut =
-                IO::Socket::INET->new( Proto => 'udp', LocalPort => $_ ) )
-            {
-                $args =
-                  ' host=' . ( $ENV{LLNGSTATUSCLIENT} || 'localhost' ) . ":$_";
-                last;
-            }
-        }
-        return $class->abort( $req,
-            "$class: status page can not be displayed, unable to open socket" )
-          unless ($statusOut);
-    }
-    return $class->abort( $req, "$class: status page can not be displayed" )
-      unless ( $statusPipe and $statusOut );
-    my $q = $req->{env}->{QUERY_STRING} || '';
-    if ( $q =~ /\s/ ) {
-        $class->logger->error("Bad characters in query");
-        return $class->FORBIDDEN;
-    }
-    $statusPipe->print(
-        "STATUS " . ( $req->{env}->{QUERY_STRING} || '' ) . "$args\n" );
-    my $buf;
-
-    while ( $_ = $statusOut->getline ) {
-        last if (/^END$/);
-        $buf .= $_;
-    }
-    $class->set_header_out( $req,
-        "Content-Type" => "text/html; charset=UTF-8" );
-    $class->print( $req, $buf );
-    return $class->OK;
-}
-
 # Method that must be called by base packages (Handler::ApacheMP2,...) to get
 # type of handler to call (Main, AuthBasic,...)
+
 sub checkType {
     my ( $class, $req ) = @_;
 
-    if ( time() - $class->lastCheck > $class->checkTime ) {
-        unless ( $class->checkConf ) {
-            $class->logger->error("$class: No configuration found");
-            $req->data->{noTry} = 1;
-            return 'Fail';
-        }
+    # Always launch "newConf" task if never started
+    msgActions->{newConf}->( $class, {}, $req )
+      unless $class->tsv and %{ $class->tsv };
+
+    # Check for event in events queue every 5 seconds
+    if ( time - $class->lastCheckMsg > $class->checkMsg ) {
+        defined( $class->checkEvent($req) ) or return 'Fail';
     }
     my $vhost = $class->resolveAlias($req);
     return ( defined $class->tsv->{type}->{$vhost} )
@@ -95,6 +51,68 @@ sub checkType {
       : 'Main';
 }
 
+# Method to check for event in events queue
+sub checkEvent {
+    my ( $class, $req, $delay ) = @_;
+    $class->logger->debug('Checking for events');
+    unless ( $class->tsv->{msgBrokerReader} ) {
+        $class->logger->error('Not initialized');
+        return;
+    }
+    my $ret = '';
+    while ( my $msg =
+        $class->tsv->{msgBrokerReader}
+        ->getNextMessage( $class->tsv->{eventQueueName}, $delay ) )
+    {
+        if ( $msg->{action} ) {
+            $class->logger->debug("Processing event $msg->{action}");
+            if ( my $sub = msgActions->{ $msg->{action} } ) {
+                $sub->( $class, $msg, $req );
+                $ret = $msg->{action};
+            }
+            else {
+                $class->logger->debug("Unknown action $msg->{action}");
+            }
+        }
+        else {
+            $class->logger->error(
+                'Malformed message: ' . JSON::to_json($msg) );
+        }
+    }
+    $class->lastCheckMsg(time);
+    return $ret;
+}
+
+# Method to push an event into message queue.
+sub publishEvent {
+    my ( $class, $req, $msg ) = @_;
+    die unless $msg;
+    $class->logger->debug("Publishing event $msg->{action}");
+    unless ( $class->tsv->{msgBrokerWriter} ) {
+        $class->logger->error('Not initialized');
+        return;
+    }
+    $class->tsv->{msgBrokerWriter}
+      ->publish( $class->tsv->{eventQueueName}, $msg );
+    my $ret;
+    my $start = time;
+
+    # After pushing message, let's pop message queue for this node and
+    # then launch the concerning method
+    $ret = $class->checkEvent( $req, 0.1 );
+    if ( !$ret or $ret ne $msg->{action} ) {
+        usleep 100000;
+        $class->checkEvent( $req, 0.1 );
+    }
+}
+
+sub publishStatus {
+    my ( $class, %msg ) = @_;
+    return unless $class->tsv->{eventStatus};
+    $class->tsv->{msgBrokerWriter}
+      ->publish( $class->tsv->{statusQueueName}, \%msg );
+}
+
 ## @rmethod int run
 # Check configuration and launch Lemonldap::NG::Handler::Main::run().
 # Each $checkTime, server child verifies if its configuration is the same
@@ -187,6 +205,7 @@ sub run {
             }
         }
         $class->set_custom( $req, $custom ) if $custom;
+        $req->userData($session);
 
         # AUTHORIZATION
         return ( $class->forbidden( $req, $session ), $session )
@@ -227,6 +246,7 @@ sub run {
         $class->cleanHeaders($req);
         return $class->OK;
     }
+
     elsif ( $protection == $class->MAYSKIP
         and $class->grant( $req, $session, $uri, $cond ) eq '999_SKIP' )
     {
@@ -240,7 +260,7 @@ sub run {
     else {
 
         # Redirect user to the portal
-        $class->logger->info("No cookie found")
+        $class->logger->debug("No cookie found")
           unless ($id);
 
         # if the cookie was fetched, a log is sent by retrieveSession()
@@ -268,13 +288,13 @@ sub unlog {
 # @param optional url string URL to log, if undefined defaults to request URI
 sub updateStatus {
     my ( $class, $req, $action, $user, $url ) = @_;
-    my $statusPipe = $class->tsv->{statusPipe} or return;
-    $user ||= $req->{env}->{REMOTE_ADDR};
+    $user ||= $req->address;
     $url  ||= $req->{env}->{REQUEST_URI};
-    eval {
-        $statusPipe->print(
-            "$user => " . $req->{env}->{HTTP_HOST} . "$url $action\n" );
-    };
+    $class->publishStatus(
+        user          => $user,
+        url           => $req->{env}->{HTTP_HOST} . $url,
+        handlerAction => $action
+    );
 }
 
 ## @rmethod void lmLog(string msg, string level)
@@ -286,6 +306,11 @@ sub lmLog {
     return $class->logger->$level($msg);
 }
 
+sub auditLog {
+    my ( $class, $req, %info ) = @_;
+    $class->_auditLogger->log( $req, %info );
+}
+
 ## @rmethod protected boolean checkMaintenanceMode
 # Check if we are in maintenance mode
 # @return true if maintenance mode is enabled
@@ -327,8 +352,7 @@ sub getLevel {
         }
     }
     if ($level) {
-        $class->logger->debug(
-            'Found AuthnLevel=' . $level . ' for "' . "$vhost$uri" . '"' );
+        $class->logger->debug("Found AuthnLevel=$level for \"$vhost$uri\"");
         return $level;
     }
     else {
@@ -345,7 +369,7 @@ sub getLevel {
 sub grant {
     my ( $class, $req, $session, $uri, $cond, $vhost ) = @_;
 
-    return $cond->( $req, $session ) if ($cond);
+    return $cond->( $req, $session ) if $cond;
 
     $vhost ||= $class->resolveAlias($req);
     my $level = $class->getLevel( $req, $uri );
@@ -397,7 +421,7 @@ sub grant {
 sub forbidden {
     my ( $class, $req, $session, $vhost ) = @_;
     my $uri    = $req->{env}->{REQUEST_URI};
-    my $portal = $class->tsv->{portal}->();
+    my $portal = $class->tsv->{portal}->($req);
     $portal = ( $portal =~ m#^https?://([^/]*).*# )[0];
     $portal =~ s/:\d+$//;
     $vhost ||= $class->resolveAlias($req);
@@ -411,15 +435,25 @@ sub forbidden {
     }
 
     if ( $session->{_upgrade} ) {
+        $class->localUnlog($req);
         return $class->goToPortal( $req, $uri, undef, '/upgradesession' );
     }
 
     # Log forbidding
-    $class->userLogger->notice( "User "
-          . $session->{ $class->tsv->{whatToTrace} }
-          . " was forbidden to access to $vhost$uri" );
+    $class->auditLog(
+        $req,
+        message => (
+                "User "
+              . $session->{ $class->tsv->{whatToTrace} }
+              . " was forbidden to access to $vhost$uri"
+        ),
+        code => "ACCESS_REFUSED",
+        user => $session->{ $class->tsv->{whatToTrace} },
+
+    );
     $class->updateStatus( $req, 'REJECT',
         $session->{ $class->tsv->{whatToTrace} } );
+    $class->localUnlog($req);
 
     # Redirect or Forbidden?
     if ( $class->tsv->{useRedirectOnForbidden} && $vhost ne $portal ) {
@@ -438,7 +472,7 @@ sub forbidden {
 sub hideCookie {
     my ( $class, $req ) = @_;
     $class->logger->debug("removing cookie");
-    my $cookie = $req->env->{HTTP_COOKIE};
+    my $cookie = $req->env->{HTTP_COOKIE} || '';
     $class->logger->debug("Cookies -> $cookie");
     my $cn = $class->tsv->{cookieName};
     $class->logger->debug("CookieName -> $cn");
@@ -469,13 +503,19 @@ sub encodeUrl {
 # @return Constant $class->REDIRECT
 sub goToPortal {
     my ( $class, $req, $url, $arg, $path ) = @_;
+
+    my $portal = $class->tsv->{portal}->($req);
+    if ( !$class->tsv->{useRedirectAjaxOnUnauthorized} and $req->wantJSON ) {
+        $class->set_header_out( $req, 'WWW-Authenticate' => "SSO $portal" );
+        return $class->AUTH_REQUIRED;
+    }
     $path ||= '';
     my ( $ret, $msg );
     my $urlc_init = $class->encodeUrl( $req, $url );
     $class->logger->debug(
-        "Redirect $req->{env}->{REMOTE_ADDR} to portal (url was $url)");
+        'Redirect ' . $req->address . " to portal (url was $url)" );
     $class->set_header_out( $req,
-            'Location' => $class->tsv->{portal}->()
+            'Location' => $portal
           . "$path?url=$urlc_init"
           . ( $arg ? "&$arg" : "" ) );
     return $class->REDIRECT;
@@ -487,7 +527,7 @@ sub goToError {
     $class->logger->debug(
         "Redirect $req->{env}->{REMOTE_ADDR} to lmError (url was $url)");
     $class->set_header_out( $req,
-            'Location' => $class->tsv->{portal}->()
+            'Location' => $class->tsv->{portal}->($req)
           . "/lmerror/$code"
           . "?url=$urlc_init" );
     return $class->REDIRECT;
@@ -504,10 +544,10 @@ sub fetchId {
       if $class->_isHttps( $req, $vhost );
     my $lookForHttpCookie = ( $class->tsv->{securedCookie} =~ /^(2|3)$/
           and not $class->_isHttps( $req, $vhost ) );
-    my $cn = $class->tsv->{cookieName};
+    my $cn    = $class->tsv->{cookieName};
     my $value = $lookForHttpCookie    # Avoid prefix and bad cookie name (#2417)
-      ? ( $t =~ /(?<![-.~])\b${cn}http=([^,; ]+)/o ? $1 : 0 )
-      : ( $t =~ /(?<![-.~])\b$cn=([^,; ]+)/o       ? $1 : 0 );
+      ? ( $t =~ /(?<![-.~])\b${cn}http=([^,; ]+)/ ? $1 : 0 )
+      : ( $t =~ /(?<![-.~])\b$cn=([^,; ]+)/       ? $1 : 0 );
 
     if ( $value && $lookForHttpCookie && $class->tsv->{securedCookie} == 3 ) {
         $value = $class->tsv->{cipher}->decryptHex( $value, "http" );
@@ -515,8 +555,14 @@ sub fetchId {
     elsif ( $value =~ s/^c:// ) {
         $value = $class->tsv->{cipher}->decrypt($value);
         unless ( $value =~ s/^(.*)? (.*)$/$1/ and $2 eq $vhost ) {
-            $class->userLogger->error(
-                "Bad CDA cookie: available for $2 instead of $vhost");
+            $class->auditLog(
+                $req,
+                message =>
+                  ("Bad CDA cookie: available for $2 instead of $vhost"),
+                code      => "CDA_BAD_HOST",
+                vhost     => $vhost,
+                cda_vhost => $2,
+            );
             return undef;
         }
     }
@@ -536,8 +582,8 @@ sub retrieveSession {
     #     (15 seconds)
     if (    defined $class->data->{_session_id}
         and $id eq $class->data->{_session_id}
-        and
-        ( $now - $class->dataUpdate < $class->tsv->{handlerInternalCache} ) )
+        and ( $now - $class->dataUpdate < $class->tsv->{handlerInternalCache} )
+      )
     {
         $class->logger->debug("Get session $id from Handler internal cache");
         return $class->data;
@@ -549,6 +595,7 @@ sub retrieveSession {
     # 2. Get the session from cache or backend
     my $session = $req->data->{session} = (
         Lemonldap::NG::Common::Session->new( {
+                hashStore            => $class->tsv->{hashedSessionStore},
                 storageModule        => $class->tsv->{sessionStorageModule},
                 storageModuleOptions => $class->tsv->{sessionStorageOptions},
                 cacheModule          => $class->tsv->{sessionCacheModule},
@@ -646,14 +693,10 @@ sub _getPort {
         return $class->tsv->{port}->{$vhost};
     }
     else {
-        if ( defined $class->tsv->{port}->{_}
-            and ( $class->tsv->{port}->{_} > 0 ) )
-        {
-            return $class->tsv->{port}->{_};
-        }
-        else {
-            return $req->port;
-        }
+        return ( defined $class->tsv->{port}->{_}
+              and ( $class->tsv->{port}->{_} > 0 ) )
+          ? $class->tsv->{port}->{_}
+          : $req->port;
     }
 }
 
@@ -670,14 +713,10 @@ sub _isHttps {
         return $class->tsv->{https}->{$vhost};
     }
     else {
-        if ( defined $class->tsv->{https}->{_}
-            and ( $class->tsv->{https}->{_} > -1 ) )
-        {
-            return $class->tsv->{https}->{_};
-        }
-        else {
-            return $req->secure;
-        }
+        return ( defined $class->tsv->{https}->{_}
+              and ( $class->tsv->{https}->{_} > -1 ) )
+          ? $class->tsv->{https}->{_}
+          : $req->secure;
     }
 }
 
@@ -698,7 +737,7 @@ sub _buildUrl {
     ) ? '' : ":$portString";
     my $url = "http" . ( $_https ? "s" : "" ) . "://$realvhost$portString$s";
     $class->logger->debug("Build URL $url");
-    return $url;
+    return URI->new($url)->as_string;
 }
 
 ## @rmethod protected int isUnprotected()
@@ -715,9 +754,8 @@ sub isUnprotected {
         $i++
       )
     {
-        if ( $uri =~ $class->tsv->{locationRegexp}->{$vhost}->[$i] ) {
-            return $class->tsv->{locationProtection}->{$vhost}->[$i];
-        }
+        return $class->tsv->{locationProtection}->{$vhost}->[$i]
+          if ( $uri =~ $class->tsv->{locationRegexp}->{$vhost}->[$i] );
     }
     return $class->tsv->{defaultProtection}->{$vhost};
 }
@@ -734,10 +772,10 @@ sub sendHeaders {
           $class->tsv->{forgeHeaders}->{$vhost}->( $req, $session );
         foreach my $h ( sort keys %headers ) {
             if ( defined( my $v = $headers{$h} ) ) {
-                $class->logger->debug("Send header $h with value $v");
+                $class->logger->debug("Send header '$h' with value '$v'");
             }
             else {
-                $class->logger->debug("Send header $h with empty value");
+                $class->logger->debug("Send header '$h' with an empty value");
             }
         }
         $class->set_header_in( $req, %headers );
@@ -785,8 +823,8 @@ sub resolveAlias {
 
     $vhost =~ s/:\d+//;
     return $class->tsv->{vhostAlias}->{$vhost}
-      if ( $class->tsv->{vhostAlias}->{$vhost} );
-    return $vhost if ( $class->tsv->{defaultCondition}->{$vhost} );
+      if $class->tsv->{vhostAlias}->{$vhost};
+    return $vhost if $class->tsv->{defaultCondition}->{$vhost};
     foreach ( @{ $class->tsv->{vhostReg} } ) {
         return $_->[1] if $vhost =~ $_->[0];
     }
@@ -841,10 +879,47 @@ sub localUnlog {
             if ( $cache->get($id) ) {
                 $cache->remove($id);
             }
+
+            # Modules like Apache::Session::REST stores hashed id in cache
+            if ( $class->tsv->{hashedSessionStore} ) {
+                $id = id2storage($id);
+                if ( $cache->get($id) ) {
+                    $cache->remove($id);
+                }
+            }
         }
     }
 }
 
+## @rmethod buildAndLoadType(string package)
+# Try to load type or build it
+my %alreadyBuilt;
+
+sub buildAndLoadType {
+    my ( $class, $package ) = @_;
+    unless ( $package =~ /^[\w:]+$/ ) {
+        $class->logger->error("Bad clas name: $package");
+        return;
+    }
+    return if $alreadyBuilt{$package};
+    eval "require $package";
+    if ($@) {
+        my $lastError = $@;
+        my $main      = $package;
+        $main =~ s/::([^:]*)$/::Main/;
+        my $type = $1 or die "Malformed package $package";
+        eval <<EOF;
+package $package;
+use base 'Lemonldap::NG::Handler::Lib::$type',
+  '$main';
+1;
+EOF
+        die "Unable to build wrapper.\n First try: $lastError\n Auto-build: $@"
+          if ($@);
+    }
+    $alreadyBuilt{$package} = 1;
+}
+
 ## @rmethod protected postOutputFilter(string uri)
 # Add a javascript to html page in order to fill html form with fake data
 # @param uri URI to catch
@@ -898,7 +973,9 @@ sub postJavascript {
     my $filler;
     foreach my $name ( keys %$data ) {
         use bytes;
-        my $value = "x" x bytes::length( $data->{$name} );
+        my @characterSet = ( '0' .. '9', 'A' .. 'Z', 'a' .. 'z' );
+        my $value        = join '' => map $characterSet[ rand @characterSet ],
+          1 .. bytes::length( $data->{$name} );
         $filler .=
 "form.find('input[name=\"$name\"], select[name=\"$name\"], textarea[name=\"$name\"]').val('$value')\n";
     }
@@ -911,7 +988,7 @@ sub postJavascript {
 
     my $jqueryUrl = $formParams->{jqueryUrl} || "";
     $jqueryUrl =
-      &{ $class->tsv->{portal} } . "static/bwr/jquery/dist/jquery.min.js"
+      $class->tsv->{portal}->($req) . "static/bwr/jquery/dist/jquery.min.js"
       if ( $jqueryUrl eq "default" );
     $jqueryUrl = "<script type='text/javascript' src='$jqueryUrl'></script>\n"
       if ($jqueryUrl);
