@@ -1,8 +1,8 @@
 package Lemonldap::NG::Handler::Main::SharedVariables;
 
-our $VERSION = '2.0.0';
+our $VERSION = '2.19.0';
 
-# Since handler has no instances but only static classes, this module provides
+# Since handler has no instance but only static classes, this module provides
 # classes properties with accessors
 
 package Lemonldap::NG::Handler::Main;
@@ -12,15 +12,19 @@ use strict;
 BEGIN {
 # Thread shared properties (if threads are available: needs to be loaded elsewhere)
     our $_tshv = {
-        tsv         => {},
-        cfgNum      => 0,
-        lastCheck   => 0,
-        checkTime   => 600,
-        confAcc     => {},
-        logger      => {},
-        userLogger  => {},
-        lmConf      => {},
-        localConfig => {},
+        tsv          => {},
+        cfgNum       => 0,
+        cfgDate      => 0,
+        checkMsg     => 5,
+        lastCheck    => 0,
+        lastCheckMsg => 0,
+        checkTime    => 600,
+        confAcc      => {},
+        logger       => {},
+        userLogger   => {},
+        _auditLogger => {},
+        lmConf       => {},
+        localConfig  => {},
     };
 
     # Current sessions properties
@@ -30,7 +34,7 @@ BEGIN {
     foreach ( keys %$_tshv ) {
         eval " sub $_ {
             my \$v = \$_[1];
-            \$_tshv->{$_} = \$v if(defined \$v);
+            \$_tshv->{$_} = \$v if (defined \$v);
             return \$_tshv->{$_};
         }";
         die $@ if ($@);
@@ -41,7 +45,7 @@ BEGIN {
     foreach ( keys %$_v ) {
         eval " sub $_ {
             my \$v = \$_[1];
-            \$_v->{$_} = \$v if(\$v);
+            \$_v->{$_} = \$v if (\$v);
             return \$_v->{$_};
         }";
         die $@ if ($@);
