@@ -26,7 +26,7 @@ use JSON 'to_json';
 use Lemonldap::NG::Common::Conf::ReConstants;
 use Lemonldap::NG::Manager::Attributes;
 
-our $VERSION = '2.21.0';
+our $VERSION = '2.22.0';
 
 extends 'Lemonldap::NG::Common::Conf::Compact';
 
@@ -373,6 +373,11 @@ sub _scanNodes {
                     $self->set( $target, [ $oldName, $key ],
                         $target, $leaf->{data} );
                 }
+                elsif ( $target =~ /^saml(?:S|ID)PMetaDataURL$/ ) {
+                    hdebug("  $target");
+                    $self->set( $target, [ $oldName, $key ],
+                        $target, $leaf->{data} );
+                }
                 elsif ( $target =~ /^saml(?:ID|S)PMetaDataOptions/ ) {
                     my $optKey = $&;
                     hdebug("  $base sub key: $target");
@@ -494,8 +499,7 @@ sub _scanNodes {
                         }
                     }
                     elsif ( $target =~
-                        /^(?:$oidcOPMetaDataNodeKeys|$oidcRPMetaDataNodeKeys)/
-                      )
+                        /^(?:$oidcOPMetaDataNodeKeys|$oidcRPMetaDataNodeKeys)/ )
                     {
                         $self->set(
                             $optKey, [ $oldName, $key ],
@@ -573,8 +577,7 @@ sub _scanNodes {
                         }
                     }
                     elsif ( $target =~
-                        /^(?:$casSrvMetaDataNodeKeys|$casAppMetaDataNodeKeys)/
-                      )
+                        /^(?:$casSrvMetaDataNodeKeys|$casAppMetaDataNodeKeys)/ )
                     {
                         $self->set(
                             $optKey, [ $oldName, $key ],
@@ -594,6 +597,30 @@ sub _scanNodes {
                 }
                 next;
             }
+            elsif ( $base eq 'keyNodes' ) {
+                hdebug('Keys');
+                hdebug("target $target");
+                hdebug("key $key");
+                if ( $target eq "KeyMaterial" ) {
+                    if ( $leaf->{data} ) {
+                        hdebug('    opened');
+                        for my $data ( @{ $leaf->{data} } ) {
+                            $self->set( "keys", $key, $data->{title},
+                                $data->{data} );
+                        }
+                    }
+                    else {
+                        hdebug('    unopened');
+                        for my $opt (qw/keyPrivate keyPrivatePwd keyPublic/) {
+                            $self->set( "keys", $key, $opt, undef );
+                        }
+                    }
+                }
+                else {
+                    $self->set( "keys", $key, $leaf->{title}, $leaf->{data} );
+                }
+                next;
+            }
             else {
                 push @{ $self->errors },
                   { message => "Fatal: unknown special sub node $base" };
