@@ -9,13 +9,15 @@ use strict;
 use Mouse;
 use JSON qw(from_json);
 use MIME::Entity;
-use Email::Sender::Simple qw(sendmail);
-use Email::Date::Format qw(email_date);
+use Lemonldap::NG::Common::EmailAddress qw(format_email);
+use Email::Sender::Simple               qw(sendmail);
+use Email::Date::Format                 qw(email_date);
+use HTML::FormatText::WithLinks;
 use Lemonldap::NG::Common::EmailTransport;
 use MIME::Base64;
 use Encode;
 
-our $VERSION = '2.0.12';
+our $VERSION = '2.21.0';
 
 our $transport;
 
@@ -42,6 +44,13 @@ has transport => (
         return $transport;
     },
 );
+has htmlParser => (
+    is      => 'ro',
+    lazy    => 1,
+    default => sub {
+        return HTML::FormatText::WithLinks->new;
+    },
+);
 
 sub loadMailTemplate {
     my ( $self, $req, $name, %prm ) = @_;
@@ -50,11 +59,6 @@ sub loadMailTemplate {
     $prm{cache}                   = 0 unless defined $prm{cache};
     $prm{params}->{STATIC_PREFIX} = $self->p->staticPrefix;
     $prm{params}->{MAIN_LOGO}     = $self->conf->{portalMainLogo};
-    my %extra =
-        $self->p->can('tplParams')
-      ? $self->p->tplParams($req)
-      : ();
-    $prm{params}->{$_} = $extra{$_} for keys %extra;
 
     return $self->loadTemplate( $req, $name, %prm );
 }
@@ -62,8 +66,7 @@ sub loadMailTemplate {
 sub translate {
     my ( $self, $req ) = @_;
 
-    # Get language using llnglanguage cookie
-    my $lang_code = $req->cookies->{llnglanguage} || 'en';
+    my $lang_code = $self->p->getLanguage($req);
     my $json      = $self->conf->{templateDir} . "/common/mail/$lang_code.json";
     $json = $self->conf->{templateDir} . '/common/mail/en.json'
       unless ( -f $json );
@@ -73,8 +76,9 @@ sub translate {
       . " ($self->{conf}->{templateDir}/$lang_code.json or $self->{conf}->{templateDir}/common/mail/en.json)";
     $json = join '', <F>;
     close F;
-    my $lang     = from_json( $json,            { allow_nonref => 1 } );
-    my $langOver = from_json( $self->p->trOver, { allow_nonref => 1 } );
+    my $lang = from_json( $json, { allow_nonref => 1 } );
+    my $langOver =
+      from_json( $self->p->getTrOver($req), { allow_nonref => 1 } );
 
     if ($langOver) {
         for my $k ( keys %{ $langOver->{all} || {} } ) {
@@ -95,10 +99,13 @@ sub translate {
 # @param regexp regular expression
 sub gen_password {
     my ( $self, $regexp ) = @_;
-    return $self->random->randregex($regexp);
+    my $password = join '',
+      sort { int( rand(3) ) - 1 } split //,
+      $self->random->randregex($regexp);
+    return $password;
 }
 
-# Send mail
+# Send mail (legacy API, using sendEmail is recommended instead)
 # @param mail recipient address
 # @param subject mail subject
 # @param body mail body
@@ -106,6 +113,11 @@ sub gen_password {
 # @return boolean result
 sub send_mail {
     my ( $self, $mail, $subject, $body, $html ) = @_;
+
+    if ( $mail =~ /^\S+\@\S+$/ ) {
+        $mail = format_email( $mail, $mail );
+    }
+
     $self->logger->info("send_mail called to send \"$subject\" to $mail");
 
     # Encode the body with the given charset
@@ -148,24 +160,33 @@ sub send_mail {
                     : ()
                 ),
                 Subject => $subject,
-                Type    => 'multipart/related',
+                Type    => 'multipart/mixed',
                 Date    => email_date,
             );
 
+            my $alternative =
+              $message->attach( Type => 'multipart/alternative' );
+
+            # Text message
+            $alternative->attach(
+                Type => 'text/plain; charset=' . $self->charset,
+                Data => $self->htmlParser->parse($body),
+            );
+
+            my $related = $alternative->attach( Type => 'multipart/related' );
+
             # Attach HTML message
-            $message->attach(
+            $related->attach(
                 Type => 'text/html; charset=' . $self->charset,
-                Data => qq{$body},
+                Data => $body,
             );
 
             # Attach included images
             foreach ( keys %cid ) {
-                $message->attach(
+                $related->attach(
                     Type => "image/" . ( $cid{$_} =~ m/\.(\w+)/ )[0],
                     Id   => $_,
-                    Path => $self->conf->{templateDir} . "/"
-                      . $self->conf->{portalSkin} . "/"
-                      . $cid{$_},
+                    Path => $self->_search_attachment( $cid{$_} ),
                 );
             }
         }
@@ -198,6 +219,24 @@ sub send_mail {
     return 1;
 }
 
+sub _search_attachment {
+    my ( $self, $filename ) = @_;
+
+    my $from_skin =
+      $self->p->getSkinTplDir( $self->conf->{portalSkin} ) . "/" . $filename;
+
+    if ( -e $from_skin ) {
+        $self->logger->debug("found attachment at $from_skin");
+        return $from_skin;
+    }
+    else {
+        my $from_bootstrap =
+          $self->conf->{templateDir} . "/bootstrap/" . $filename;
+        $self->logger->debug("returning attachment at $from_bootstrap");
+        return $from_bootstrap;
+    }
+}
+
 ## @method string getMailSession(string user)
 # Check if a mail session exists
 # @param user the value of the user key in session
@@ -215,7 +254,7 @@ sub getMailSession {
     # Browse found sessions to check if it's a mail session
     foreach my $id ( keys %$sessions ) {
         my $mailSession =
-          $self->p->getApacheSession( $id, ( kind => "TOKEN" ) );
+          $self->p->getApacheSession( $id, kind => "TOKEN", hashStore => 0, );
         next unless ($mailSession);
         return $mailSession if ( $mailSession->data->{_type} =~ /^mail$/ );
     }
@@ -241,7 +280,7 @@ sub getRegisterSession {
     # Browse found sessions to check if it's a register session
     foreach my $id ( keys %$sessions ) {
         my $registerSession =
-          $self->p->getApacheSession( $id, ( kind => "TOKEN" ) );
+          $self->p->getApacheSession( $id, kind => "TOKEN", hashStore => 0, );
         next unless ($registerSession);
         return $id
           if (  $registerSession->data->{_type}
@@ -252,4 +291,80 @@ sub getRegisterSession {
     return "";
 }
 
+## @method string sendEmail(string mail)
+# High-level method for sending a templated email
+# Takes a hash of parameters
+# @param dest the destination email address
+# @param subject the plaintext subject, will not be translated
+# @param subject_trmsg the translation key that will be used as subject
+# @param body the plaintext body, may contain $variable placeholders
+# @param body_template the name of the template file to render as body
+# @param params a hashref of parameters
+# @return success status as a boolean
+sub sendEmail {
+    my ( $self, $req, %params ) = @_;
+
+    my $subject_trmsg = $params{subject_trmsg};
+    my $subject       = $params{subject};
+    my $body          = $params{body};
+    my $body_template = $params{body_template};
+    my %variables     = %{ $params{params} || {} };
+    my $dest          = $params{dest};
+
+    # Build mail content
+    my $tr = $self->translate($req);
+    unless ($subject) {
+        $subject = $subject_trmsg;
+        $tr->( \$subject );
+    }
+    my $html;
+    if ($body) {
+
+        # Replace supplied variables in body, and also handle session variables
+        while ( my ( $k, $v ) = each %variables ) {
+            $body =~ s/\$\Q$k\E\b/$v/g;
+        }
+        $body =~ s/\$(\w+)/$req->{sessionInfo}->{$1} || ''/ge;
+    }
+    else {
+
+        # Use HTML template
+        $body = $self->loadMailTemplate(
+            $req,
+            $body_template,
+            filter => $tr,
+            params => \%variables
+        );
+        $html = 1;
+    }
+    if ( $dest && $subject && $body ) {
+        if ( $self->send_mail( $dest, $subject, $body, $html ) ) {
+            $self->auditLog(
+                $req,
+                message => "Email ($body_template) sent "
+                  . "to $dest with subject '$subject'",
+                code          => "EMAIL_SENT",
+                body_template => $body_template,
+                subject       => $subject,
+                subject_trmsg => $subject_trmsg,
+                dest          => $dest,
+            );
+            return 1;
+        }
+        else {
+            return 0;
+        }
+    }
+    else {
+        my $message = "Could not send email:";
+        $message .= " missing destination," unless $dest;
+        $message .= " missing subject,"     unless $subject;
+        $message .= " missing body,"        unless $body;
+        chop($message);
+        $message .= ".";
+        $self->logger->warn($message);
+        return 0;
+    }
+}
+
 1;
