@@ -47,11 +47,11 @@
   };
   displayError = function displayError(j, status, err) {
     var refresh, res;
-    console.log('Error', err);
+    console.error('Error', err);
     res = JSON.parse(j.responseText);
     if (res && res.error) {
       res = res.error.replace(/.* /, '');
-      console.log('Returned error', res);
+      console.error('Returned error', res);
       if (res.match(/module/)) {
         return setMsg('notAuthorized', 'warning');
       } else if (res === 'csrfToken') {
